package jalview.commands;
  /*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2006 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

import jalview.datamodel.*;

public class RemoveGapsCommand  extends EditCommand
{
  public RemoveGapsCommand(  String description,
                             SequenceI[] seqs, char gapChar)
  {
    this.description = description;
    int width = 0;
    for(int i=0; i<seqs.length; i++)
      if(seqs[i].getLength()>width)
        width = seqs[i].getLength();

    findGaps(seqs, 0, width, gapChar);
  }

  public RemoveGapsCommand(  String description,
                             SequenceI[] seqs,
                             int start, int end, char gapChar)
  {
    this.description = description;
    findGaps(seqs, start, end, gapChar);
  }

  void findGaps(SequenceI [] seqs, int start, int end, char gapChar)
  {

    int startCol = -1, endCol = -1;
    int deletedCols = 0;

    edits = new Edit[0];

    boolean delete = true;
    char [] sequence;
    for(int s=0; s<seqs.length; s++)
    {
      deletedCols = 0;
      startCol = -1;
      endCol = -1;
      sequence = seqs[s].getSequence().toCharArray();
      for (int i = start; i < end; i++)
      {
        delete = true;

        if (!jalview.util.Comparison.isGap(sequence[i]))
        {
          if (delete)
            endCol = i;

          delete = false;
        }

        if (delete && startCol == -1)
        {
          startCol = i;
        }

        if (!delete && startCol > -1)
        {
          this.appendEdit(DELETE_GAP, new SequenceI[]{seqs[s]},
                          startCol - deletedCols,
                          endCol - startCol,
                          gapChar,
                          false);

          deletedCols += (endCol - startCol);
          startCol = -1;
          endCol = -1;
        }
      }
      if (delete && startCol > -1)
      {
        int width = end - endCol;

        if (endCol == -1)
          width = end - start + 1;

        //This is the end of the region.
        this.appendEdit(DELETE_GAP, new SequenceI[]{seqs[s]},
                        startCol - deletedCols,
                        width,
                        gapChar,
                        false);
      }

    }

    performEdit(0);
  }

}
